@@ -59,7 +59,9 @@ struct _GthSlideshowPrivate {
 	ClutterTimeline       *timeline;
 	ClutterActor          *image1;
 	ClutterActor          *image2;
+	ClutterActor          *paused_actor;
 #endif
+	GdkPixbuf             *current_pixbuf;
 	GtkWidget             *viewer;
 	guint                  next_event;
 	guint                  delay;
@@ -73,6 +75,7 @@ struct _GthSlideshowPrivate {
 	int                    current_audio_file;
 	GstElement            *playbin;
 #endif
+	GdkPixbuf             *pause_pixbuf;
 	gboolean               paused;
 	gboolean               animating;
 	gboolean               random_order;
@@ -125,6 +128,9 @@ _gth_slideshow_load_current_image (GthSlideshow *self)
 	GthFileData *requested_file;
 	GthFileData *next_file;
 	GthFileData *prev_file;
+	int          screen_width;
+	int          screen_height;
+	GdkScreen   *screen;
 
 	if (self->priv->next_event != 0) {
 		g_source_remove (self->priv->next_event);
@@ -148,10 +154,21 @@ _gth_slideshow_load_current_image (GthSlideshow *self)
 		prev_file = (GthFileData *) self->priv->current->prev->data;
 	else
 		prev_file = NULL;
+
+	screen_width = -1;
+	screen_height = -1;
+	screen = gtk_widget_get_screen (GTK_WIDGET (self));
+	if (screen != NULL) {
+		screen_width = gdk_screen_get_width (screen);
+		screen_height = gdk_screen_get_height (screen);
+	}
+
 	gth_image_preloader_load (self->priv->preloader,
 				  requested_file,
+				  MAX (screen_width, screen_height),
 				  next_file,
-				  prev_file);
+				  prev_file,
+				  NULL);
 }
 
 
@@ -210,13 +227,16 @@ view_next_image_automatically (GthSlideshow *self)
 
 
 static void
-image_preloader_requested_ready_cb (GthImagePreloader *preloader,
-				    GError            *error,
-				    gpointer           user_data)
+image_preloader_requested_ready_cb (GthImagePreloader  *preloader,
+				    GthFileData        *requested,
+				    GdkPixbufAnimation *animation,
+				    int                 original_width,
+				    int                 original_height,
+				    GError             *error,
+				    gpointer            user_data)
 {
-	GthSlideshow   *self = user_data;
-	GthImageLoader *image_loader;
-	GdkPixbuf      *pixbuf;
+	GthSlideshow *self = user_data;
+	GdkPixbuf    *static_image;
 
 	if (error != NULL) {
 		g_clear_error (&error);
@@ -224,20 +244,21 @@ image_preloader_requested_ready_cb (GthImagePreloader *preloader,
 		return;
 	}
 
-	image_loader = gth_image_preloader_get_loader (self->priv->preloader, (GthFileData *) self->priv->current->data);
-	if (image_loader == NULL) {
-		_gth_slideshow_load_next_image (self);
-		return;
-	}
+	_g_object_unref (self->priv->current_pixbuf);
 
-	pixbuf = gth_image_loader_get_pixbuf (image_loader);
-	if (pixbuf == NULL) {
+	static_image = gdk_pixbuf_animation_get_static_image (animation);
+	if (static_image != NULL)
+		self->priv->current_pixbuf = gdk_pixbuf_copy (static_image);
+	else
+		self->priv->current_pixbuf = NULL;
+
+	if (self->priv->current_pixbuf == NULL) {
 		_gth_slideshow_load_next_image (self);
 		return;
 	}
 
 	self->priv->one_loaded = TRUE;
-	self->priv->projector->image_ready (self, pixbuf);
+	self->priv->projector->image_ready (self, self->priv->current_pixbuf);
 }
 
 
@@ -259,8 +280,9 @@ gth_slideshow_init (GthSlideshow *self)
 	self->priv->animating = FALSE;
 	self->priv->direction = GTH_SLIDESHOW_DIRECTION_FORWARD;
 	self->priv->random_order = FALSE;
+	self->priv->current_pixbuf = NULL;
 
-	self->priv->preloader = gth_image_preloader_new ();
+	self->priv->preloader = gth_image_preloader_new (GTH_LOAD_POLICY_ONE_STEP, 3);
 	g_signal_connect (self->priv->preloader,
 			  "requested_ready",
 			  G_CALLBACK (image_preloader_requested_ready_cb),
@@ -278,6 +300,8 @@ gth_slideshow_finalize (GObject *object)
 	if (self->priv->hide_cursor_event != 0)
 		g_source_remove (self->priv->hide_cursor_event);
 
+	_g_object_unref (self->priv->pause_pixbuf);
+	_g_object_unref (self->priv->current_pixbuf);
 	_g_object_list_unref (self->priv->file_list);
 	_g_object_unref (self->priv->browser);
 	_g_object_unref (self->priv->preloader);
@@ -441,7 +465,11 @@ _gth_slideshow_construct (GthSlideshow *self,
 	self->priv->browser = _g_object_ref (browser);
 	self->priv->file_list = _g_object_list_ref (file_list);
 	self->priv->one_loaded = FALSE;
-
+	self->priv->pause_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+							     "slideshow-pause",
+							     100,
+							     0,
+							     NULL);
 	self->priv->projector->construct (self);
 
 	g_signal_connect (self, "show", G_CALLBACK (gth_slideshow_show_cb), self);
@@ -536,7 +564,7 @@ static void
 default_projector_image_ready (GthSlideshow *self,
 			       GdkPixbuf    *pixbuf)
 {
-	gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer), pixbuf);
+	gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->viewer), pixbuf, -1, -1);
 	view_next_image_automatically (self);
 }
 
@@ -558,7 +586,7 @@ default_projector_hide_cursor (GthSlideshow *self)
 static void
 default_projector_paused (GthSlideshow *self)
 {
-	/* void */
+	gtk_widget_queue_draw (self->priv->viewer);
 }
 
 
@@ -585,22 +613,34 @@ viewer_event_cb (GtkWidget    *widget,
 			break;
 		}
 	}
+	else if (event->type == GDK_KEY_PRESS) {
+		switch (((GdkEventKey *) event)->keyval) {
+		case GDK_F5:
+			_gth_slideshow_close (self);
+			break;
+		}
+	}
 	else if (event->type == GDK_KEY_RELEASE) {
 		switch (((GdkEventKey *) event)->keyval) {
 		case GDK_Escape:
+		case GDK_q:
 			_gth_slideshow_close (self);
 			break;
-
-		case GDK_space:
+		case GDK_p:
 			_gth_slideshow_toggle_pause (self);
 			break;
 
+		case GDK_space:
 		case GDK_Down:
 		case GDK_Right:
 		case GDK_Page_Down:
-			_gth_slideshow_load_next_image (self);
+			if (self->priv->paused)
+				_gth_slideshow_toggle_pause (self);
+			else
+				_gth_slideshow_load_next_image (self);
 			break;
 
+		case GDK_BackSpace:
 		case GDK_Up:
 		case GDK_Left:
 		case GDK_Page_Up:
@@ -611,6 +651,32 @@ viewer_event_cb (GtkWidget    *widget,
 }
 
 
+static void
+default_projector_pause_painter (GthImageViewer *image_viewer,
+				 GdkEventExpose *event,
+				 cairo_t        *cr,
+				 gpointer        user_data)
+{
+	GthSlideshow *self = user_data;
+	GdkScreen    *screen;
+	double        dest_x;
+	double        dest_y;
+
+	if (! self->priv->paused || (self->priv->pause_pixbuf == NULL))
+		return;
+
+	screen = gtk_widget_get_screen (GTK_WIDGET (image_viewer));
+	if (screen == NULL)
+		return;
+
+	dest_x = gdk_screen_get_width (screen) - gdk_pixbuf_get_width (self->priv->pause_pixbuf) - 10.0;
+	dest_y = gdk_screen_get_height (screen) - gdk_pixbuf_get_height (self->priv->pause_pixbuf) - 10.0;
+	gdk_cairo_set_source_pixbuf (cr, self->priv->pause_pixbuf, dest_x, dest_y);
+	cairo_rectangle (cr, dest_x, dest_y, gdk_pixbuf_get_width (self->priv->pause_pixbuf), gdk_pixbuf_get_height (self->priv->pause_pixbuf));
+	cairo_fill (cr);
+}
+
+
 static void
 default_projector_construct (GthSlideshow *self)
 {
@@ -621,9 +687,11 @@ default_projector_construct (GthSlideshow *self)
 	gth_image_viewer_set_zoom_change (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_ZOOM_CHANGE_FIT_SIZE);
 	gth_image_viewer_set_transp_type (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_TRANSP_TYPE_BLACK);
 	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_ZOOM_QUALITY_LOW);
+	gth_image_viewer_add_painter (GTH_IMAGE_VIEWER (self->priv->viewer), default_projector_pause_painter, self);
 
 	g_signal_connect (self->priv->viewer, "button-press-event", G_CALLBACK (viewer_event_cb), self);
 	g_signal_connect (self->priv->viewer, "motion-notify-event", G_CALLBACK (viewer_event_cb), self);
+	g_signal_connect (self->priv->viewer, "key-press-event", G_CALLBACK (viewer_event_cb), self);
 	g_signal_connect (self->priv->viewer, "key-release-event", G_CALLBACK (viewer_event_cb), self);
 
 	gtk_widget_show (self->priv->viewer);
@@ -883,10 +951,28 @@ clutter_projector_hide_cursor (GthSlideshow *self)
 static void
 clutter_projector_paused (GthSlideshow *self)
 {
+	float stage_w;
+	float stage_h;
+
 	if (self->priv->animating) {
 		clutter_timeline_pause (self->priv->timeline);
 		_gth_slideshow_animation_completed (self);
 	}
+
+	clutter_actor_get_size (self->stage, &stage_w, &stage_h);
+	clutter_actor_set_position (self->priv->paused_actor, stage_w / 2.0, stage_h / 2.0);
+	clutter_actor_set_anchor_point_from_gravity (self->priv->paused_actor, CLUTTER_GRAVITY_CENTER);
+	clutter_actor_set_scale (self->priv->paused_actor, 1.0, 1.0);
+	clutter_actor_set_opacity (self->priv->paused_actor, 255);
+	clutter_actor_raise_top (self->priv->paused_actor);
+	clutter_actor_show (self->priv->paused_actor);
+
+	clutter_actor_animate (self->priv->paused_actor,
+			       CLUTTER_LINEAR, 500,
+	                       "opacity", 0,
+	                       "scale-x", 3.0,
+	                       "scale-y", 3.0,
+	                       NULL);
 }
 
 
@@ -913,25 +999,37 @@ stage_input_cb (ClutterStage *stage,
 			break;
 		}
 	}
+	else if (event->type == CLUTTER_KEY_PRESS) {
+		switch (clutter_event_get_key_symbol (event)) {
+		case CLUTTER_F5:
+			_gth_slideshow_close (self);
+			break;
+		}
+	}
 	else if (event->type == CLUTTER_KEY_RELEASE) {
 		switch (clutter_event_get_key_symbol (event)) {
 		case CLUTTER_Escape:
+		case CLUTTER_q:
 			_gth_slideshow_close (self);
 			break;
-
-		case CLUTTER_space:
+		case CLUTTER_p:
 			_gth_slideshow_toggle_pause (self);
 			break;
 
-		case CLUTTER_Up:
+		case CLUTTER_space:
+		case CLUTTER_Down:
 		case CLUTTER_Right:
-		case CLUTTER_Page_Up:
-			_gth_slideshow_load_next_image (self);
+		case CLUTTER_Page_Down:
+			if (self->priv->paused)
+				_gth_slideshow_toggle_pause (self);
+			else
+				_gth_slideshow_load_next_image (self);
 			break;
 
-		case CLUTTER_Down:
+		case CLUTTER_BackSpace:
+		case CLUTTER_Up:
 		case CLUTTER_Left:
-		case CLUTTER_Page_Down:
+		case CLUTTER_Page_Up:
 			_gth_slideshow_load_prev_image (self);
 			break;
 		}
@@ -940,14 +1038,9 @@ stage_input_cb (ClutterStage *stage,
 
 
 static void
-gth_slideshow_size_allocate_cb (GtkWidget     *widget,
-				GtkAllocation *allocation,
-				gpointer       user_data)
+adapt_image_size_to_stage_size (GthSlideshow *self)
 {
-	GthSlideshow   *self = user_data;
 	gfloat          stage_w, stage_h;
-	GthImageLoader *image_loader;
-	GdkPixbuf      *pixbuf;
 	GdkPixbuf      *image;
 	int             pixbuf_w, pixbuf_h;
 	int             pixbuf_x, pixbuf_y;
@@ -960,28 +1053,23 @@ gth_slideshow_size_allocate_cb (GtkWidget     *widget,
 	if ((stage_w == 0) || (stage_h == 0))
 		return;
 
-	image_loader = gth_image_preloader_get_loader (self->priv->preloader, (GthFileData *) self->priv->current->data);
-	if (image_loader == NULL)
+	if (self->priv->current_pixbuf == NULL)
 		return;
 
-	pixbuf = gth_image_loader_get_pixbuf (image_loader);
-	if (pixbuf == NULL)
-		return;
-
-	image = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (pixbuf),
+	image = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (self->priv->current_pixbuf),
 				FALSE,
-				gdk_pixbuf_get_bits_per_sample (pixbuf),
+				gdk_pixbuf_get_bits_per_sample (self->priv->current_pixbuf),
 				stage_w,
 				stage_h);
 	gdk_pixbuf_fill (image, 0x000000ff);
 
-	pixbuf_w = gdk_pixbuf_get_width (pixbuf);
-	pixbuf_h = gdk_pixbuf_get_height (pixbuf);
+	pixbuf_w = gdk_pixbuf_get_width (self->priv->current_pixbuf);
+	pixbuf_h = gdk_pixbuf_get_height (self->priv->current_pixbuf);
 	scale_keeping_ratio (&pixbuf_w, &pixbuf_h, (int) stage_w, (int) stage_h, TRUE);
 	pixbuf_x = (stage_w - pixbuf_w) / 2;
 	pixbuf_y = (stage_h - pixbuf_h) / 2;
 
-	gdk_pixbuf_composite (pixbuf,
+	gdk_pixbuf_composite (self->priv->current_pixbuf,
 			      image,
 			      pixbuf_x,
 			      pixbuf_y,
@@ -989,8 +1077,8 @@ gth_slideshow_size_allocate_cb (GtkWidget     *widget,
 			      pixbuf_h,
 			      pixbuf_x,
 			      pixbuf_y,
-			      (double) pixbuf_w / gdk_pixbuf_get_width (pixbuf),
-			      (double) pixbuf_h / gdk_pixbuf_get_height (pixbuf),
+			      (double) pixbuf_w / gdk_pixbuf_get_width (self->priv->current_pixbuf),
+			      (double) pixbuf_h / gdk_pixbuf_get_height (self->priv->current_pixbuf),
 			      GDK_INTERP_BILINEAR,
 			      255);
 
@@ -1010,6 +1098,15 @@ gth_slideshow_size_allocate_cb (GtkWidget     *widget,
 }
 
 
+static void
+gth_slideshow_size_allocate_cb (GtkWidget     *widget,
+				GtkAllocation *allocation,
+				gpointer       user_data)
+{
+	adapt_image_size_to_stage_size (GTH_SLIDESHOW (user_data));
+}
+
+
 static void
 clutter_projector_construct (GthSlideshow *self)
 {
@@ -1022,11 +1119,9 @@ clutter_projector_construct (GthSlideshow *self)
 	clutter_stage_set_color (CLUTTER_STAGE (self->stage), &background_color);
 	g_signal_connect (self->stage, "button-press-event", G_CALLBACK (stage_input_cb), self);
 	g_signal_connect (self->stage, "motion-event", G_CALLBACK (stage_input_cb), self);
+	g_signal_connect (self->stage, "key-press-event", G_CALLBACK (stage_input_cb), self);
 	g_signal_connect (self->stage, "key-release-event", G_CALLBACK (stage_input_cb), self);
 
-	gtk_widget_show (embed);
-	gtk_container_add (GTK_CONTAINER (self), embed);
-
 	self->priv->image1 = clutter_texture_new ();
 	clutter_actor_hide (self->priv->image1);
 	clutter_container_add_actor (CLUTTER_CONTAINER (self->stage), self->priv->image1);
@@ -1043,7 +1138,19 @@ clutter_projector_construct (GthSlideshow *self)
 	g_signal_connect (self->priv->timeline, "new-frame", G_CALLBACK (animation_frame_cb), self);
 	g_signal_connect (self->priv->timeline, "started", G_CALLBACK (animation_started_cb), self);
 
+	if (self->priv->pause_pixbuf != NULL)
+		self->priv->paused_actor = gtk_clutter_texture_new_from_pixbuf (self->priv->pause_pixbuf);
+	else
+		self->priv->paused_actor = gtk_clutter_texture_new_from_stock (GTK_WIDGET (self),
+									       GTK_STOCK_MEDIA_PAUSE,
+									       GTK_ICON_SIZE_DIALOG);
+	clutter_actor_hide (self->priv->paused_actor);
+	clutter_container_add_actor (CLUTTER_CONTAINER (self->stage), self->priv->paused_actor);
+
 	g_signal_connect (self, "size-allocate", G_CALLBACK (gth_slideshow_size_allocate_cb), self);
+
+	gtk_widget_show (embed);
+	gtk_container_add (GTK_CONTAINER (self), embed);
 }
 
 
