@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -33,6 +32,49 @@
 
 #define GET_WIDGET(name) _gtk_builder_get_widget (data->builder, (name))
 #define DEFAULT_ICON "application-x-executable"
+#define EXTENSION_CATEGORY_ALL "*"
+#define EXTENSION_CATEGORY_ENABLED "+"
+#define EXTENSION_CATEGORY_DISABLED "-"
+#define EXTENSION_CATEGORY_SEPARATOR "---"
+
+
+enum {
+	EXTENSION_DESCRIPTION_COLUMN,
+	EXTENSION_ORIGINAL_STATUS_COLUMN,
+	EXTENSION_COLUMNS
+};
+
+
+enum {
+	CATEGORY_ID_COLUMN,
+	CATEGORY_NAME_COLUMN,
+	CATEGORY_ICON_COLUMN,
+	CATEGORY_SEPARATOR_COLUMN,
+	CATEGORY_COLUMNS
+};
+
+
+typedef struct {
+	const char *id;
+	const char *name;
+	const char *icon;
+} ExtensionCategory;
+
+
+static ExtensionCategory extension_category[] = {
+		{ EXTENSION_CATEGORY_ALL, N_("All"), "folder" },
+		{ EXTENSION_CATEGORY_ENABLED, N_("Enabled"), "folder" },
+		{ EXTENSION_CATEGORY_DISABLED, N_("Disabled"), "folder" },
+		{ EXTENSION_CATEGORY_SEPARATOR, NULL, NULL },
+		{ "Browser", N_("Browser"), "folder" },
+		{ "Viewer", N_("Viewers"), "folder" },
+		{ "Metadata", N_("Metadata"), "folder" },
+		{ "File-Tool", N_("File tools"), "folder" },
+		{ "List-Tool", N_("List tools"), "folder" },
+		{ "Importer", N_("Importers"), "folder" },
+		{ "Exporter", N_("Exporters"), "folder" },
+		{ NULL, NULL, NULL }
+};
 
 
 typedef struct {
@@ -41,7 +83,9 @@ typedef struct {
 	GtkWidget    *dialog;
 	GtkWidget    *list_view;
 	GtkListStore *list_store;
+	GtkTreeModel *model_filter;
 	GSList       *active_extensions;
+	char         *current_category;
 } DialogData;
 
 
@@ -91,6 +135,9 @@ destroy_cb (GtkWidget  *widget,
 		GthExtensionDescription *description;
 
 		description = gth_extension_manager_get_description (manager, name);
+		if ((description == NULL) || description->mandatory || description->hidden)
+			continue;
+
 		if (gth_extension_description_is_active (description))
 			active_extensions = g_slist_prepend (active_extensions, g_strdup (name));
 	}
@@ -124,6 +171,7 @@ destroy_cb (GtkWidget  *widget,
 	g_slist_foreach (data->active_extensions, (GFunc) g_free, NULL);
 	g_slist_free (data->active_extensions);
 	g_object_unref (data->builder);
+	g_free (data->current_category);
 	g_free (data);
 }
 
@@ -198,22 +246,24 @@ cell_renderer_toggle_toggled_cb (GtkCellRendererToggle *cell_renderer,
 	if (tree_path == NULL)
 		return;
 
-	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (data->list_store), &iter, tree_path)) {
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (data->model_filter), &iter, tree_path)) {
 		GthExtensionDescription *description;
 		GError                  *error = NULL;
+		GtkTreeIter              child_iter;
 
-		gtk_tree_model_get (GTK_TREE_MODEL (data->list_store), &iter, 0, &description, -1);
+		gtk_tree_model_get (GTK_TREE_MODEL (data->model_filter), &iter, 0, &description, -1);
+		gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (data->model_filter), &child_iter, &iter);
 		if (! gth_extension_description_is_active (description)) {
 			if (! gth_extension_manager_activate (gth_main_get_default_extension_manager (), description->id, &error))
 				_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), _("Could not activate the extension"), &error);
 			else
-				gtk_list_store_set (data->list_store, &iter, 0, description, -1);
+				gtk_list_store_set (data->list_store, &child_iter, 0, description, -1);
 		}
 		else {
 			if (! gth_extension_manager_deactivate (gth_main_get_default_extension_manager (), description->id, &error))
 				_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->dialog), _("Could not deactivate the extension"), &error);
 			else
-				gtk_list_store_set (data->list_store, &iter, 0, description, -1);
+				gtk_list_store_set (data->list_store, &child_iter, 0, description, -1);
 		}
 
 		g_object_unref (description);
@@ -298,11 +348,11 @@ list_view_selection_changed_cb (GtkTreeSelection *treeselection,
 	GthExtensionDescription *description;
 	GthExtension            *extension;
 
-	model = GTK_TREE_MODEL (data->list_store);
+	model = GTK_TREE_MODEL (data->model_filter);
 	if (! gtk_tree_selection_get_selected (treeselection, &model, &iter))
 		return;
 
-	gtk_tree_model_get (model, &iter, 0, &description, -1);
+	gtk_tree_model_get (model, &iter, EXTENSION_DESCRIPTION_COLUMN, &description, -1);
 
 	extension = gth_extension_description_get_extension (description);
 	gtk_widget_set_sensitive (GET_WIDGET ("preferences_button"), (extension != NULL) && gth_extension_is_configurable (extension));
@@ -311,6 +361,51 @@ list_view_selection_changed_cb (GtkTreeSelection *treeselection,
 }
 
 
+static void
+reset_original_extension_status (DialogData *data)
+{
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+
+	model = GTK_TREE_MODEL (data->list_store);
+	if (! gtk_tree_model_get_iter_first (model, &iter))
+		return;
+
+	do {
+		GthExtensionDescription *description;
+
+		gtk_tree_model_get (model, &iter, EXTENSION_DESCRIPTION_COLUMN, &description, -1);
+		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+				    EXTENSION_ORIGINAL_STATUS_COLUMN, gth_extension_description_is_active (description),
+				    -1);
+
+		g_object_unref (description);
+	}
+	while (gtk_tree_model_iter_next (model, &iter));
+}
+
+
+static void
+category_treeview_selection_changed_cb (GtkTreeSelection *treeselection,
+					gpointer          user_data)
+{
+	DialogData   *data = user_data;
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+
+	model = GTK_TREE_MODEL (data->model_filter);
+	if (! gtk_tree_selection_get_selected (treeselection, &model, &iter))
+		return;
+
+	reset_original_extension_status (data);
+
+	g_free (data->current_category);
+	gtk_tree_model_get (model, &iter, CATEGORY_ID_COLUMN, &data->current_category, -1);
+
+	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (data->model_filter));
+}
+
+
 static void
 about_button_clicked_cb (GtkButton *button,
 			 gpointer   user_data)
@@ -321,7 +416,7 @@ about_button_clicked_cb (GtkButton *button,
 	GthExtensionDescription *description;
 	GtkWidget               *dialog;
 
-	model = GTK_TREE_MODEL (data->list_store);
+	model = GTK_TREE_MODEL (data->model_filter);
 	if (! gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (data->list_view)), &model, &iter))
 		return;
 
@@ -353,6 +448,70 @@ about_button_clicked_cb (GtkButton *button,
 }
 
 
+static void
+preferences_button_clicked_cb (GtkButton *button,
+			       gpointer   user_data)
+{
+	DialogData              *data = user_data;
+	GtkTreeModel            *model;
+	GtkTreeIter              iter;
+	GthExtensionDescription *description;
+	GthExtension            *extension;
+
+	model = GTK_TREE_MODEL (data->model_filter);
+	if (! gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (data->list_view)), &model, &iter))
+		return;
+
+	gtk_tree_model_get (model, &iter, 0, &description, -1);
+	extension = gth_extension_description_get_extension (description);
+	gth_extension_configure (extension, GTK_WINDOW (data->dialog));
+
+	g_object_unref (description);
+}
+
+
+static gboolean
+category_view_separator_func (GtkTreeModel *model,
+			      GtkTreeIter  *iter,
+			      gpointer      data)
+{
+	gboolean separator;
+
+	gtk_tree_model_get (model, iter, CATEGORY_SEPARATOR_COLUMN, &separator, -1);
+	return separator;
+}
+
+
+static gboolean
+category_model_visible_func (GtkTreeModel *model,
+			     GtkTreeIter  *iter,
+			     gpointer      user_data)
+{
+	DialogData              *data = user_data;
+	GthExtensionDescription *description;
+	gboolean                 original_status_is_active;
+	gboolean                 visible;
+
+	if (strcmp (data->current_category, EXTENSION_CATEGORY_ALL) == 0)
+		return TRUE;
+
+	gtk_tree_model_get (model, iter,
+			    EXTENSION_DESCRIPTION_COLUMN, &description,
+			    EXTENSION_ORIGINAL_STATUS_COLUMN, &original_status_is_active,
+			    -1);
+
+	visible = FALSE;
+	if ((strcmp (data->current_category, EXTENSION_CATEGORY_ENABLED) == 0) && original_status_is_active)
+		visible = TRUE;
+	else if ((strcmp (data->current_category, EXTENSION_CATEGORY_DISABLED) == 0) && ! original_status_is_active)
+		visible = TRUE;
+	else
+		visible = g_strcmp0 (description->category, data->current_category) == 0;
+
+	return visible;
+}
+
+
 void
 dlg_extensions (GthBrowser *browser)
 {
@@ -360,17 +519,37 @@ dlg_extensions (GthBrowser *browser)
 	GthExtensionManager *manager;
 	GList               *extensions;
 	GList               *scan;
+	GSList              *all_active_extensions;
+	GSList              *s_scan;
 	GtkTreePath         *first;
+	int                  i;
 
 	if (gth_browser_get_dialog (browser, "extensions") != NULL) {
 		gtk_window_present (GTK_WINDOW (gth_browser_get_dialog (browser, "extensions")));
 		return;
 	}
 
+	manager = gth_main_get_default_extension_manager ();
+
 	data = g_new0 (DialogData, 1);
 	data->browser = browser;
 	data->builder = _gtk_builder_new_from_file ("extensions.ui", NULL);
-	data->active_extensions = eel_gconf_get_string_list (PREF_ACTIVE_EXTENSIONS);
+
+	data->active_extensions = NULL;
+	all_active_extensions = eel_gconf_get_string_list (PREF_ACTIVE_EXTENSIONS);
+	for (s_scan = all_active_extensions; s_scan; s_scan = s_scan->next) {
+		char                    *name = s_scan->data;
+		GthExtensionDescription *description;
+
+		description = gth_extension_manager_get_description (manager, name);
+		if ((description == NULL) || description->mandatory || description->hidden)
+			continue;
+
+		data->active_extensions = g_slist_prepend (data->active_extensions, g_strdup (name));
+	}
+	data->active_extensions = g_slist_reverse (data->active_extensions);
+	g_slist_foreach (all_active_extensions, (GFunc) g_free, NULL);
+	g_slist_free (all_active_extensions);
 
 	/* Get the widgets. */
 
@@ -380,8 +559,10 @@ dlg_extensions (GthBrowser *browser)
 
 	/* Set widgets data. */
 
-	data->list_store = gtk_list_store_new (1, G_TYPE_OBJECT);
-	data->list_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (data->list_store));
+	data->list_store = gtk_list_store_new (EXTENSION_COLUMNS, G_TYPE_OBJECT, G_TYPE_BOOLEAN);
+	data->model_filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (data->list_store), NULL);
+	data->list_view = gtk_tree_view_new_with_model (data->model_filter);
+	g_object_unref (data->model_filter);
 	g_object_unref (data->list_store);
         gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (data->list_view), TRUE);
         gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (data->list_view), TRUE);
@@ -404,14 +585,48 @@ dlg_extensions (GthBrowser *browser)
 		GtkTreeIter              iter;
 
 		description = gth_extension_manager_get_description (manager, name);
-		if ((description == NULL) || description->mandatory)
+		if ((description == NULL) || description->mandatory || description->hidden)
 			continue;
 
 		gtk_list_store_append (data->list_store, &iter);
-		gtk_list_store_set (data->list_store, &iter, 0, description, -1);
+		gtk_list_store_set (data->list_store, &iter,
+				    EXTENSION_DESCRIPTION_COLUMN, description,
+				    EXTENSION_ORIGINAL_STATUS_COLUMN, gth_extension_description_is_active (description),
+				    -1);
 	}
 	g_list_free (extensions);
 
+	/* the category list */
+
+	data->current_category = g_strdup (EXTENSION_CATEGORY_ALL);
+	gtk_tree_selection_set_mode (GTK_TREE_SELECTION (gtk_tree_view_get_selection (GTK_TREE_VIEW (GET_WIDGET ("category_treeview")))), GTK_SELECTION_BROWSE);
+	for (i = 0; extension_category[i].id != NULL; i++) {
+		GtkTreeIter iter;
+
+		gtk_list_store_append (GTK_LIST_STORE (GET_WIDGET ("category_liststore")), &iter);
+		if (strcmp (extension_category[i].id, EXTENSION_CATEGORY_SEPARATOR) == 0)
+			gtk_list_store_set (GTK_LIST_STORE (GET_WIDGET ("category_liststore")),
+					    &iter,
+					    CATEGORY_SEPARATOR_COLUMN, TRUE,
+					    -1);
+		else
+			gtk_list_store_set (GTK_LIST_STORE (GET_WIDGET ("category_liststore")),
+					    &iter,
+					    CATEGORY_NAME_COLUMN, _(extension_category[i].name),
+					    CATEGORY_ID_COLUMN, extension_category[i].id,
+					    /* CATEGORY_ICON_COLUMN, extension_category[i].icon, */
+					    CATEGORY_SEPARATOR_COLUMN, FALSE,
+					    -1);
+	}
+	gtk_tree_view_set_row_separator_func (GTK_TREE_VIEW (GET_WIDGET ("category_treeview")),
+					      category_view_separator_func,
+					      data,
+					      NULL);
+	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (data->model_filter),
+						category_model_visible_func,
+						data,
+						NULL);
+
 	/* Set the signals handlers. */
 
 	g_signal_connect (G_OBJECT (data->dialog),
@@ -426,10 +641,19 @@ dlg_extensions (GthBrowser *browser)
 			  "clicked",
 			  G_CALLBACK (about_button_clicked_cb),
 			  data);
+	g_signal_connect (GET_WIDGET ("preferences_button"),
+			  "clicked",
+			  G_CALLBACK (preferences_button_clicked_cb),
+			  data);
 	g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (data->list_view)),
 			  "changed",
 			  G_CALLBACK (list_view_selection_changed_cb),
 			  data);
+	g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (GET_WIDGET ("category_treeview"))),
+			  "changed",
+			  G_CALLBACK (category_treeview_selection_changed_cb),
+			  data);
+
 	first = gtk_tree_path_new_first ();
 	gtk_tree_selection_select_path (gtk_tree_view_get_selection (GTK_TREE_VIEW (data->list_view)), first);
 	gtk_tree_path_free (first);
