@@ -16,8 +16,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <string.h>
@@ -582,7 +581,7 @@ g_directory_foreach_child (GFile                *directory,
 						      g_str_equal,
 						      g_free,
 						      NULL);
-	fec->metadata_attributes = ! _g_file_attributes_matches_any (fec->attributes, GIO_ATTRIBUTES);
+	fec->metadata_attributes = ! _g_file_attributes_matches_all (fec->attributes, GIO_ATTRIBUTES);
 
 	g_file_query_info_async (fec->base_directory,
 				 fec->attributes,
@@ -882,8 +881,7 @@ g_directory_list_async (GFile             *directory,
 
 typedef struct {
 	GList             *file_list;
-	gboolean           recursive;
-	gboolean           follow_links;
+	GthListFlags       flags;
 	char              *attributes;
 	GCancellable      *cancellable;
 	InfoReadyCallback  callback;
@@ -938,8 +936,14 @@ query_data__for_each_file_cb (GFile     *file,
 {
 	QueryInfoData *query_data = user_data;
 
-	if (g_file_info_get_file_type (info) != G_FILE_TYPE_DIRECTORY)
-		query_data->files = g_list_prepend (query_data->files, gth_file_data_new (file, info));
+	if (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY)
+		return;
+	if ((query_data->flags & GTH_LIST_NO_BACKUP_FILES) && g_file_info_get_is_backup (info))
+		return;
+	if ((query_data->flags & GTH_LIST_NO_HIDDEN_FILES) && g_file_info_get_is_hidden (info))
+		return;
+
+	query_data->files = g_list_prepend (query_data->files, gth_file_data_new (file, info));
 }
 
 
@@ -951,6 +955,11 @@ query_data__start_dir_cb (GFile       *directory,
 {
 	QueryInfoData *query_data = user_data;
 
+	if ((query_data->flags & GTH_LIST_NO_BACKUP_FILES) && g_file_info_get_is_backup (info))
+		return DIR_OP_SKIP;
+	if ((query_data->flags & GTH_LIST_NO_HIDDEN_FILES) && g_file_info_get_is_hidden (info))
+		return DIR_OP_SKIP;
+
 	query_data->files = g_list_prepend (query_data->files, gth_file_data_new (directory, info));
 
 	return DIR_OP_CONTINUE;
@@ -973,10 +982,10 @@ query_data_info_ready_cb (GObject      *source_object,
 		return;
 	}
 
-	if (query_data->recursive && (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY)) {
+	if ((query_data->flags & GTH_LIST_RECURSIVE) && (g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY)) {
 		g_directory_foreach_child ((GFile *) query_data->current->data,
 					   TRUE,
-					   query_data->follow_links,
+					   (query_data->flags & GTH_LIST_NO_FOLLOW_LINKS) == 0,
 					   query_data->attributes,
 					   query_data->cancellable,
 					   query_data__start_dir_cb,
@@ -996,6 +1005,8 @@ query_data_info_ready_cb (GObject      *source_object,
 static void
 query_info__query_current (QueryInfoData *query_data)
 {
+	GFileQueryInfoFlags flags;
+
 	if (query_data->current == NULL) {
 		query_data->files = g_list_reverse (query_data->files);
 		query_data->callback (query_data->files, NULL, query_data->user_data);
@@ -1003,9 +1014,13 @@ query_info__query_current (QueryInfoData *query_data)
 		return;
 	}
 
+	flags = G_FILE_QUERY_INFO_NONE;
+	if (query_data->flags & GTH_LIST_NO_FOLLOW_LINKS)
+		flags |= G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS;
+
 	g_file_query_info_async ((GFile *) query_data->current->data,
 				 query_data->attributes,
-				 (query_data->follow_links ? G_FILE_QUERY_INFO_NONE : G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS),
+				 flags,
 				 G_PRIORITY_DEFAULT,
 				 query_data->cancellable,
 				 query_data_info_ready_cb,
@@ -1015,8 +1030,7 @@ query_info__query_current (QueryInfoData *query_data)
 
 void
 _g_query_info_async (GList             *file_list,
-		     gboolean           recursive,
-		     gboolean           follow_links,
+		     GthListFlags       flags,
 		     const char        *attributes,
 		     GCancellable      *cancellable,
 		     InfoReadyCallback  ready_callback,
@@ -1026,9 +1040,11 @@ _g_query_info_async (GList             *file_list,
 
 	query_data = g_new0 (QueryInfoData, 1);
 	query_data->file_list = _g_object_list_ref (file_list);
-	query_data->recursive = recursive;
-	query_data->follow_links = follow_links;
-	query_data->attributes = g_strconcat (attributes, ",standard::name,standard::type,id::file", NULL);
+	query_data->flags = flags;
+	query_data->attributes = g_strconcat ("standard::name,standard::type,standard::is-hidden,standard::is-backup,id::file",
+					      (((attributes != NULL) && (strcmp (attributes, "") != 0)) ? "," : NULL),
+					      attributes,
+					      NULL);
 	query_data->cancellable = _g_object_ref (cancellable);
 	query_data->callback = ready_callback;
 	query_data->user_data = user_data;
@@ -1085,30 +1101,30 @@ _g_dummy_file_op_async (ReadyFunc callback,
 
 
 typedef struct {
-	GthFileData      *source;
-	GFile            *destination;
-	gboolean          move;
-	GFileCopyFlags    flags;
-	int               io_priority;
-	goffset           tot_size;
-	goffset           copied_size;
-	gsize             tot_files;
-	GCancellable     *cancellable;
-	ProgressCallback  progress_callback;
-	gpointer          progress_callback_data;
-	DialogCallback    dialog_callback;
-	gpointer          dialog_callback_data;
-	ReadyFunc         ready_callback;
-	gpointer          user_data;
+	GthFileData          *source;
+	GFile                *destination;
+	gboolean              move;
+	GFileCopyFlags        flags;
+	int                   io_priority;
+	goffset               tot_size;
+	goffset               copied_size;
+	gsize                 tot_files;
+	GCancellable         *cancellable;
+	ProgressCallback      progress_callback;
+	gpointer              progress_callback_data;
+	DialogCallback        dialog_callback;
+	gpointer              dialog_callback_data;
+	CopyReadyCallback     ready_callback;
+	gpointer              user_data;
 
-	GFile            *current_destination;
-	char             *message;
-	int               default_response;
+	GFile                *current_destination;
+	char                 *message;
+	GthOverwriteResponse  default_response;
 
-	GList            *source_sidecars;  /* GFile list */
-	GList            *destination_sidecars;  /* GFile list */
-	GList            *current_source_sidecar;
-	GList            *current_destination_sidecar;
+	GList                *source_sidecars;  /* GFile list */
+	GList                *destination_sidecars;  /* GFile list */
+	GList                *current_source_sidecar;
+	GList                *current_destination_sidecar;
 } CopyFileData;
 
 
@@ -1132,14 +1148,14 @@ copy_file__delete_source (CopyFileData *copy_file_data)
 	GError *error = NULL;
 
 	if (! copy_file_data->move) {
-		copy_file_data->ready_callback (NULL, copy_file_data->user_data);
+		copy_file_data->ready_callback (copy_file_data->default_response, NULL, copy_file_data->user_data);
 		copy_file_data_free (copy_file_data);
 		return;
 	}
 
 	g_file_delete (copy_file_data->source->file, copy_file_data->cancellable, &error);
 
-	copy_file_data->ready_callback (error, copy_file_data->user_data);
+	copy_file_data->ready_callback (copy_file_data->default_response, error, copy_file_data->user_data);
 	copy_file_data_free (copy_file_data);
 }
 
@@ -1221,22 +1237,29 @@ copy_file__overwrite_dialog_response_cb (GtkDialog *dialog,
 	CopyFileData *copy_file_data = user_data;
 
 	if (response_id != GTK_RESPONSE_OK)
-		copy_file_data->default_response = GTH_OVERWRITE_RESPONSE_UNSPECIFIED;
+		copy_file_data->default_response = GTH_OVERWRITE_RESPONSE_CANCEL;
 	else
 		copy_file_data->default_response = gth_overwrite_dialog_get_response (GTH_OVERWRITE_DIALOG (dialog));
 
 	gtk_widget_hide (GTK_WIDGET (dialog));
 
 	if (copy_file_data->dialog_callback != NULL)
-		copy_file_data->dialog_callback (FALSE, copy_file_data->dialog_callback_data);
+		copy_file_data->dialog_callback (FALSE, NULL, copy_file_data->dialog_callback_data);
 
 	switch (copy_file_data->default_response) {
 	case GTH_OVERWRITE_RESPONSE_NO:
 	case GTH_OVERWRITE_RESPONSE_ALWAYS_NO:
 	case GTH_OVERWRITE_RESPONSE_UNSPECIFIED:
-		copy_file_data->ready_callback (NULL, copy_file_data->user_data);
-		copy_file_data_free (copy_file_data);
-		return;
+	case GTH_OVERWRITE_RESPONSE_CANCEL:
+		{
+			GError *error = NULL;
+
+			if (copy_file_data->default_response == GTH_OVERWRITE_RESPONSE_CANCEL)
+				error = g_error_new_literal (G_IO_ERROR, G_IO_ERROR_CANCELLED, "");
+			copy_file_data->ready_callback (copy_file_data->default_response, error, copy_file_data->user_data);
+			copy_file_data_free (copy_file_data);
+			return;
+		}
 
 	case GTH_OVERWRITE_RESPONSE_YES:
 	case GTH_OVERWRITE_RESPONSE_ALWAYS_YES:
@@ -1272,24 +1295,31 @@ copy_file_ready_cb (GObject      *source_object,
 
 	if (! g_file_copy_finish ((GFile *) source_object, result, &error)) {
 		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
-			GtkWidget *dialog;
+			if (copy_file_data->default_response != GTH_OVERWRITE_RESPONSE_ALWAYS_NO) {
+				GtkWidget *dialog;
 
-			if (copy_file_data->dialog_callback != NULL)
-				copy_file_data->dialog_callback (TRUE, copy_file_data->dialog_callback_data);
+				dialog = gth_overwrite_dialog_new (copy_file_data->source->file,
+								   NULL,
+								   copy_file_data->current_destination,
+								   copy_file_data->default_response,
+								   copy_file_data->tot_files == 1);
 
-			dialog = gth_overwrite_dialog_new (copy_file_data->source->file,
-							   copy_file_data->current_destination,
-							   copy_file_data->default_response,
-							   copy_file_data->tot_files == 1);
-			g_signal_connect (dialog,
-					  "response",
-					  G_CALLBACK (copy_file__overwrite_dialog_response_cb),
-					  copy_file_data);
-			gtk_widget_show (dialog);
+				if (copy_file_data->dialog_callback != NULL)
+					copy_file_data->dialog_callback (TRUE, dialog, copy_file_data->dialog_callback_data);
 
+				g_signal_connect (dialog,
+						  "response",
+						  G_CALLBACK (copy_file__overwrite_dialog_response_cb),
+						  copy_file_data);
+				gtk_widget_show (dialog);
+			}
+			else {
+				copy_file_data->ready_callback (copy_file_data->default_response, error, copy_file_data->user_data);
+				copy_file_data_free (copy_file_data);
+			}
 			return;
 		}
-		copy_file_data->ready_callback (error, copy_file_data->user_data);
+		copy_file_data->ready_callback (copy_file_data->default_response, error, copy_file_data->user_data);
 		copy_file_data_free (copy_file_data);
 		return;
 	}
@@ -1352,6 +1382,9 @@ _g_copy_file_to_destination (CopyFileData   *copy_file_data,
 	_g_object_unref (copy_file_data->current_destination);
 	copy_file_data->current_destination = g_file_dup (destination);
 
+	if (copy_file_data->default_response == GTH_OVERWRITE_RESPONSE_ALWAYS_YES)
+		flags = G_FILE_COPY_OVERWRITE;
+
 	if (copy_file_data->progress_callback != NULL) {
 		GFile *destination_parent;
 		char  *destination_name;
@@ -1383,7 +1416,7 @@ _g_copy_file_to_destination (CopyFileData   *copy_file_data,
 			if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS))
 				g_clear_error (&error);
 		}
-		copy_file_data->ready_callback (error, copy_file_data->user_data);
+		copy_file_data->ready_callback (copy_file_data->default_response, error, copy_file_data->user_data);
 		copy_file_data_free (copy_file_data);
 		return;
 	}
@@ -1400,11 +1433,12 @@ _g_copy_file_to_destination (CopyFileData   *copy_file_data,
 }
 
 
-void
+static void
 _g_copy_file_async_private (GthFileData           *source,
 			    GFile                 *destination,
 			    gboolean               move,
 			    GFileCopyFlags         flags,
+			    GthOverwriteResponse   default_response,
 			    int                    io_priority,
 			    goffset                tot_size,
 			    goffset                copied_size,
@@ -1414,7 +1448,7 @@ _g_copy_file_async_private (GthFileData           *source,
 			    gpointer               progress_callback_data,
 			    DialogCallback         dialog_callback,
 			    gpointer               dialog_callback_data,
-			    ReadyFunc              ready_callback,
+			    CopyReadyCallback      ready_callback,
 			    gpointer               user_data)
 {
 	CopyFileData *copy_file_data;
@@ -1435,6 +1469,7 @@ _g_copy_file_async_private (GthFileData           *source,
 	copy_file_data->dialog_callback_data = dialog_callback_data;
 	copy_file_data->ready_callback = ready_callback;
 	copy_file_data->user_data = user_data;
+	copy_file_data->default_response = default_response;
 
 	_g_copy_file_to_destination (copy_file_data, copy_file_data->destination, 0);
 }
@@ -1445,19 +1480,21 @@ _g_copy_file_async (GthFileData           *source,
 		    GFile                 *destination,
 		    gboolean               move,
 		    GFileCopyFlags         flags,
+		    GthOverwriteResponse   default_response,
 		    int                    io_priority,
 		    GCancellable          *cancellable,
 		    ProgressCallback       progress_callback,
 		    gpointer               progress_callback_data,
 		    DialogCallback         dialog_callback,
 		    gpointer               dialog_callback_data,
-		    ReadyFunc              ready_callback,
+		    CopyReadyCallback      ready_callback,
 		    gpointer               user_data)
 {
 	_g_copy_file_async_private (source,
 				    destination,
 				    move,
 				    flags,
+				    default_response,
 				    io_priority,
 				    g_file_info_get_size (source->info),
 				    0,
@@ -1476,37 +1513,37 @@ _g_copy_file_async (GthFileData           *source,
 
 
 typedef struct {
-	GHashTable        *source_hash;
-	GFile             *destination;
+	GHashTable           *source_hash;
+	GFile                *destination;
 
-	GList             *files;  /* GthFileData list */
-	GList             *current;
-	GFile             *source_base;
-	GFile             *current_destination;
+	GList                *files;  /* GthFileData list */
+	GList                *current;
+	GFile                *source_base;
+	GFile                *current_destination;
 
-	GList             *source_sidecars;  /* GFile list */
-	GList             *destination_sidecars;  /* GFile list */
-	GList             *current_source_sidecar;
-	GList             *current_destination_sidecar;
+	GList                *source_sidecars;  /* GFile list */
+	GList                *destination_sidecars;  /* GFile list */
+	GList                *current_source_sidecar;
+	GList                *current_destination_sidecar;
 
-	goffset            tot_size;
-	goffset            copied_size;
-	gsize              tot_files;
-	gsize              copied_files;
+	goffset               tot_size;
+	goffset               copied_size;
+	gsize                 tot_files;
+	gsize                 copied_files;
 
-	char              *message;
-	int                default_response;
+	char                 *message;
+	GthOverwriteResponse  default_response;
 
-	gboolean           move;
-	GFileCopyFlags     flags;
-	int                io_priority;
-	GCancellable      *cancellable;
-	ProgressCallback   progress_callback;
-	gpointer           progress_callback_data;
-	DialogCallback     dialog_callback;
-	gpointer           dialog_callback_data;
-	ReadyFunc          done_callback;
-	gpointer           user_data;
+	gboolean              move;
+	GFileCopyFlags        flags;
+	int                   io_priority;
+	GCancellable         *cancellable;
+	ProgressCallback      progress_callback;
+	gpointer              progress_callback_data;
+	DialogCallback        dialog_callback;
+	gpointer              dialog_callback_data;
+	ReadyFunc             done_callback;
+	gpointer              user_data;
 } CopyData;
 
 
@@ -1566,8 +1603,9 @@ copy_data__copy_next_file (CopyData *copy_data)
 
 
 static void
-copy_data__copy_current_file_ready_cb (GError   *error,
-			 	       gpointer  user_data)
+copy_data__copy_current_file_ready_cb (GthOverwriteResponse  response,
+				       GError               *error,
+			 	       gpointer              user_data)
 {
 	CopyData *copy_data = user_data;
 
@@ -1577,6 +1615,7 @@ copy_data__copy_current_file_ready_cb (GError   *error,
 		return;
 	}
 
+	copy_data->default_response = response;
 	copy_data__copy_next_file (copy_data);
 }
 
@@ -1608,6 +1647,7 @@ copy_data__copy_current_file (CopyData *copy_data)
 				    destination,
 				    FALSE,
 				    flags,
+				    copy_data->default_response,
 				    copy_data->io_priority,
 				    copy_data->tot_size,
 				    copy_data->copied_size,
@@ -1684,6 +1724,7 @@ _g_copy_files_async (GList            *sources, /* GFile list */
 	copy_data->dialog_callback_data = dialog_callback_data;
 	copy_data->done_callback = done_callback;
 	copy_data->user_data = user_data;
+	copy_data->default_response = GTH_OVERWRITE_RESPONSE_UNSPECIFIED;
 
 	copy_data->source_hash = g_hash_table_new_full ((GHashFunc) g_file_hash, (GEqualFunc) g_file_equal, (GDestroyNotify) g_object_unref, NULL);
 	for (scan = sources; scan; scan = scan->next)
@@ -1698,8 +1739,7 @@ _g_copy_files_async (GList            *sources, /* GFile list */
 					      copy_data->progress_callback_data);
 
 	_g_query_info_async (sources,
-			     TRUE,
-			     TRUE,
+			     GTH_LIST_RECURSIVE,
 			     "standard::name,standard::display-name,standard::type,standard::size",
 			     copy_data->cancellable,
 			     copy_files__sources_info_ready_cb,
@@ -1851,7 +1891,8 @@ _g_delete_files_async (GList        *file_list,
 		       ReadyFunc     callback,
 		       gpointer      user_data)
 {
-	DeleteData *delete_data;
+	DeleteData   *delete_data;
+	GthListFlags  flags;
 
 	delete_data = g_new0 (DeleteData, 1);
 	delete_data->include_metadata = include_metadata;
@@ -1859,9 +1900,12 @@ _g_delete_files_async (GList        *file_list,
 	delete_data->callback = callback;
 	delete_data->user_data = user_data;
 
+	flags = GTH_LIST_NO_FOLLOW_LINKS;
+	if (recursive)
+		flags |= GTH_LIST_RECURSIVE;
+
 	_g_query_info_async (file_list,
-			     recursive,
-			     FALSE,
+			     flags,
 			     GFILE_NAME_TYPE_ATTRIBUTES,
 			     delete_data->cancellable,
 			     delete_files__info_ready_cb,
@@ -1906,6 +1950,8 @@ g_load_file_in_buffer (GFile   *file,
 			break;
 		}
 
+		/* the '+ 1' here is used to allow to add a NULL character at the
+		 * end of the buffer. */
 		local_buffer = g_realloc (local_buffer, count + n + 1);
 		memcpy (local_buffer + count, tmp_buffer, n);
 		count += n;
@@ -1950,14 +1996,14 @@ load_file__stream_read_cb (GObject      *source_object,
 
 	count = g_input_stream_read_finish (load_data->stream, result, &error);
 	if (count < 0) {
-		load_data->callback (NULL, -1, error, load_data->user_data);
+		load_data->callback (&load_data->buffer, -1, error, load_data->user_data);
 		load_data_free (load_data);
 		return;
 	}
 	else if (count == 0) {
 		if (load_data->buffer != NULL)
 			((char *)load_data->buffer)[load_data->count] = 0;
-		load_data->callback (load_data->buffer, load_data->count, NULL, load_data->user_data);
+		load_data->callback (&load_data->buffer, load_data->count, NULL, load_data->user_data);
 		load_data_free (load_data);
 		return;
 	}
@@ -1986,7 +2032,7 @@ load_file__file_read_cb (GObject      *source_object,
 
 	load_data->stream = (GInputStream *) g_file_read_finish (G_FILE (source_object), result, &error);
 	if (load_data->stream == NULL) {
-		load_data->callback (NULL, -1, error, load_data->user_data);
+		load_data->callback (&load_data->buffer, -1, error, load_data->user_data);
 		load_data_free (load_data);
 		return;
 	}
@@ -2039,6 +2085,7 @@ typedef struct {
 static void
 write_data_free (WriteData *write_data)
 {
+	g_free (write_data->buffer);
 	g_free (write_data);
 }
 
@@ -2048,7 +2095,7 @@ write_file__notify (gpointer user_data)
 {
 	WriteData *write_data = user_data;
 
-	write_data->callback (write_data->buffer, write_data->count, write_data->error, write_data->user_data);
+	write_data->callback (&write_data->buffer, write_data->count, write_data->error, write_data->user_data);
 	write_data_free (write_data);
 }
 
@@ -2113,7 +2160,34 @@ write_file__replace_ready_cb (GObject      *source_object,
 
 	stream = (GOutputStream*) g_file_replace_finish ((GFile*) source_object, result, &error);
 	if (stream == NULL) {
-		write_data->callback (write_data->buffer, write_data->count, error, write_data->user_data);
+		write_data->callback (&write_data->buffer, write_data->count, error, write_data->user_data);
+		write_data_free (write_data);
+		return;
+	}
+
+	write_data->written = 0;
+	g_output_stream_write_async (stream,
+				     write_data->buffer,
+				     write_data->count,
+				     write_data->io_priority,
+				     write_data->cancellable,
+				     write_file__stream_write_ready_cb,
+				     write_data);
+}
+
+
+static void
+write_file__create_ready_cb (GObject      *source_object,
+			     GAsyncResult *result,
+			     gpointer      user_data)
+{
+	WriteData     *write_data = user_data;
+	GOutputStream *stream;
+	GError        *error = NULL;
+
+	stream = (GOutputStream*) g_file_create_finish ((GFile*) source_object, result, &error);
+	if (stream == NULL) {
+		write_data->callback (&write_data->buffer, write_data->count, error, write_data->user_data);
 		write_data_free (write_data);
 		return;
 	}
@@ -2133,6 +2207,7 @@ void
 g_write_file_async (GFile               *file,
 		    void                *buffer,
 		    gsize                count,
+		    gboolean             replace,
 		    int                  io_priority,
 		    GCancellable        *cancellable,
 		    BufferReadyCallback  callback,
@@ -2148,7 +2223,22 @@ g_write_file_async (GFile               *file,
 	write_data->callback = callback;
 	write_data->user_data = user_data;
 
-	g_file_replace_async (file, NULL, FALSE, 0, io_priority, cancellable, write_file__replace_ready_cb, write_data);
+	if (replace)
+		g_file_replace_async (file,
+				      NULL,
+				      FALSE,
+				      0,
+				      io_priority,
+				      cancellable,
+				      write_file__replace_ready_cb,
+				      write_data);
+	else
+		g_file_create_async (file,
+				     0,
+				     io_priority,
+				     cancellable,
+				     write_file__create_ready_cb,
+				     write_data);
 }
 
 
@@ -2161,7 +2251,7 @@ _g_file_create_unique (GFile       *parent,
 	GFile             *file = NULL;
 	GError            *local_error = NULL;
 	int                n;
-	GFileOutputStream *stream;
+	GFileOutputStream *stream = NULL;
 
 	file = g_file_get_child_for_display_name (parent, display_name, &local_error);
 	n = 0;
@@ -2252,6 +2342,39 @@ _g_directory_create_unique (GFile       *parent,
 }
 
 
+#define MAX_ATTEMPS 10
+
+
+GFile *
+_g_directory_create_tmp (void)
+{
+	GFile *tmp_dir;
+	GFile *dir = NULL;
+	int    n;
+
+	tmp_dir = g_file_new_for_path (g_get_tmp_dir ());
+	if (tmp_dir == NULL)
+		return NULL;
+
+	for (n = 0; n < MAX_ATTEMPS; n++) {
+		char  *name;
+
+		name = _g_rand_string (12);
+		dir = g_file_get_child (tmp_dir, name);
+		g_free (name);
+
+		if (g_file_make_directory (dir, NULL, NULL))
+			break;
+
+		g_object_unref (dir);
+	}
+
+	g_object_unref (tmp_dir);
+
+	return dir;
+}
+
+
 /* -- g_write_file -- */
 
 
@@ -2297,3 +2420,22 @@ _g_directory_make (GFile    *file,
 					    NULL,
 					    error);
 }
+
+
+gboolean
+_g_file_set_modification_time (GFile         *file,
+			       GTimeVal      *timeval,
+			       GCancellable  *cancellable,
+			       GError       **error)
+{
+	GFileInfo *info;
+	gboolean   result;
+
+	info = g_file_info_new ();
+        g_file_info_set_modification_time (info, timeval);
+        result = g_file_set_attributes_from_info (file, info, G_FILE_QUERY_INFO_NONE, cancellable, error);
+
+        g_object_unref (info);
+
+        return result;
+}
