@@ -86,7 +86,7 @@ gth_cell_renderer_thumbnail_get_property (GObject    *object,
 {
 	GthCellRendererThumbnail *self;
 
-	self = GTH_CELL_RENDERER_THUMBNAIL (object);
+	self = (GthCellRendererThumbnail *) object;
 
 	switch (param_id) {
 	case PROP_SIZE:
@@ -125,7 +125,7 @@ gth_cell_renderer_thumbnail_set_property (GObject      *object,
 {
 	GthCellRendererThumbnail *self;
 
-	self = GTH_CELL_RENDERER_THUMBNAIL (object);
+	self = (GthCellRendererThumbnail *) object;
 
 	switch (param_id) {
 	case PROP_SIZE:
@@ -173,7 +173,7 @@ gth_cell_renderer_thumbnail_get_size (GtkCellRenderer *cell,
   	int   xpad;
   	int   ypad;
 
-  	self = GTH_CELL_RENDERER_THUMBNAIL (cell);
+  	self = (GthCellRendererThumbnail *) cell;
 
   	if (self->priv->thumbnail != NULL) {
   		image_width = gdk_pixbuf_get_width (self->priv->thumbnail);
@@ -186,7 +186,11 @@ gth_cell_renderer_thumbnail_get_size (GtkCellRenderer *cell,
 
   	gtk_cell_renderer_get_padding (cell, &xpad, &ypad);
 
-	if (self->priv->is_icon || self->priv->fixed_size || (self->priv->thumbnail == NULL) || ((image_width < self->priv->size) && (image_height < self->priv->size))) {
+	if (self->priv->is_icon
+	    || self->priv->fixed_size
+	    || (self->priv->thumbnail == NULL)
+	    || ((image_width < self->priv->size) && (image_height < self->priv->size)))
+	{
 		calc_width  = (int) (xpad * 2) + (THUMBNAIL_X_BORDER * 2) + self->priv->size;
 		calc_height = (int) (ypad * 2) + (THUMBNAIL_Y_BORDER * 2) + self->priv->size;
 	}
@@ -319,7 +323,7 @@ gth_cell_renderer_thumbnail_render (GtkCellRenderer      *cell,
 	GdkPixbuf                *colorized = NULL;
 	int                       border;
 
-	self = GTH_CELL_RENDERER_THUMBNAIL (cell);
+	self = (GthCellRendererThumbnail *) cell;
 
  	gth_cell_renderer_thumbnail_get_size (cell, widget, cell_area,
  					      &thumb_rect.x,
@@ -383,22 +387,14 @@ gth_cell_renderer_thumbnail_render (GtkCellRenderer      *cell,
 		if (state == GTK_STATE_ACTIVE)
 			state = GTK_STATE_SELECTED;
 
-		if (! _g_mime_type_is_image (gth_file_data_get_mime_type (self->priv->file))) {
+		if (! _g_mime_type_is_image (gth_file_data_get_mime_type (self->priv->file)))
 			border = 1;
-			frame_rect = image_rect;
-		}
-		else {
+		else
 			border = 4;
 
-			/*frame_rect = thumb_rect;
-			frame_rect.width = self->priv->size;
-			frame_rect.height = self->priv->size;
-			frame_rect.x = thumb_rect.x + (thumb_rect.width - frame_rect.width) * .5;
-			frame_rect.y = thumb_rect.y + (thumb_rect.height - frame_rect.height) * .5;*/
-
-			frame_rect = image_rect;
-		}
+		frame_rect = image_rect;
 
+		cairo_translate (cr, 0.5, 0.5);
 		cairo_set_line_width (cr, 0.5);
 		cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
 		gdk_cairo_set_source_color (cr, &style->dark[state]);
@@ -406,34 +402,34 @@ gth_cell_renderer_thumbnail_render (GtkCellRenderer      *cell,
 		/* the outer frame */
 
 		cairo_rectangle (cr,
-				 frame_rect.x - border + 0.5,
-				 frame_rect.y - border + 0.5,
+				 frame_rect.x - border,
+				 frame_rect.y - border,
 				 frame_rect.width + (border * 2) - 1,
 				 frame_rect.height + (border * 2) - 1);
 
 		/* the drop shadow */
 
 		cairo_move_to (cr,
-			       frame_rect.x - (border / 2) + 0.5,
-			       frame_rect.y + frame_rect.height + border + 0.5);
+			       frame_rect.x - (border / 2),
+			       frame_rect.y + frame_rect.height + border);
 		cairo_line_to (cr,
-			       frame_rect.x + frame_rect.width + border + 0.5,
-			       frame_rect.y + frame_rect.height + border + 0.5);
+			       frame_rect.x + frame_rect.width + border,
+			       frame_rect.y + frame_rect.height + border);
 		cairo_line_to (cr,
-			       frame_rect.x + frame_rect.width + border + 0.5,
-			       frame_rect.y - (border / 2) + 0.5);
-
+			       frame_rect.x + frame_rect.width + border,
+			       frame_rect.y - (border / 2));
 		cairo_move_to (cr,
-			       frame_rect.x - (border / 2) + 0.5,
-			       frame_rect.y + frame_rect.height + border + 1.5);
+			       frame_rect.x - (border / 2),
+			       frame_rect.y + frame_rect.height + border + 1.0);
 		cairo_line_to (cr,
-			       frame_rect.x + frame_rect.width + border + 1.5,
-			       frame_rect.y + frame_rect.height + border + 1.5);
+			       frame_rect.x + frame_rect.width + border + 1.0,
+			       frame_rect.y + frame_rect.height + border + 1.0);
 		cairo_line_to (cr,
-			       frame_rect.x + frame_rect.width + border + 1.5,
-			       frame_rect.y - (border / 2) + 0.5);
-
+			       frame_rect.x + frame_rect.width + border + 1.0,
+			       frame_rect.y - (border / 2));
 		cairo_stroke (cr);
+
+		cairo_identity_matrix (cr);
 	}
 
   	if (! self->priv->checked || ((flags & (GTK_CELL_RENDERER_SELECTED | GTK_CELL_RENDERER_PRELIT)) != 0)) {
